@@ -14,16 +14,12 @@ BEGIN
 
 use SDL::App::FPS::MyOpenGL;
 
-my $options = { 
- width => 640, height => 480, depth => 16, gl => 1, max_fps => 0
- };
-
 print
   "SDL::App::FPS OpenGL Demo v0.01 (C) 2002,2003 by Tels <http://Bloodgate.com/>\n\n";
 
 print "Mouse buttons for speed changes, f for fullscreen and q for quit.\n";
 
-my $app = SDL::App::FPS::MyOpenGL->new( $options );
+my $app = SDL::App::FPS::MyOpenGL->new( config => 'config/opengl.cfg' );
 $app->main_loop();
 
 print "Running time was ", int($app->now() / 10)/100, " seconds\n";
