@@ -32,12 +32,12 @@ sub _init
   }
 
 my $remap = {
-  SDLK_LSHIFT() => SDL::KMOD_LSHIFT,
-  SDLK_RSHIFT() => SDL::KMOD_RSHIFT,
-  SDLK_LCTRL() => SDL::KMOD_LCTRL,
-  SDLK_RCTRL() => SDL::KMOD_RCTRL,
-  SDLK_LALT() => SDL::KMOD_LALT,
-  SDLK_RALT() => SDL::KMOD_RALT,
+  SDLK_LSHIFT() => KMOD_LSHIFT,
+  SDLK_RSHIFT() => KMOD_RSHIFT,
+  SDLK_LCTRL() => KMOD_LCTRL,
+  SDLK_RCTRL() => KMOD_RCTRL,
+  SDLK_LALT() => KMOD_LALT,
+  SDLK_RALT() => KMOD_RALT,
   };
 
 sub _init_mod
@@ -203,32 +203,32 @@ my $char2key = {
   y => SDLK_y,
   z => SDLK_z,
   
-  A => [ SDLK_a, SDL::KMOD_SHIFT ],
-  B => [ SDLK_b, SDL::KMOD_SHIFT ],
-  C => [ SDLK_c, SDL::KMOD_SHIFT ],
-  D => [ SDLK_d, SDL::KMOD_SHIFT ],
-  E => [ SDLK_e, SDL::KMOD_SHIFT ],
-  F => [ SDLK_f, SDL::KMOD_SHIFT ],
-  G => [ SDLK_g, SDL::KMOD_SHIFT ],
-  H => [ SDLK_h, SDL::KMOD_SHIFT ],
-  I => [ SDLK_i, SDL::KMOD_SHIFT ],
-  J => [ SDLK_j, SDL::KMOD_SHIFT ],
-  K => [ SDLK_k, SDL::KMOD_SHIFT ],
-  L => [ SDLK_l, SDL::KMOD_SHIFT ],
-  M => [ SDLK_m, SDL::KMOD_SHIFT ],
-  N => [ SDLK_n, SDL::KMOD_SHIFT ],
-  O => [ SDLK_o, SDL::KMOD_SHIFT ],
-  P => [ SDLK_p, SDL::KMOD_SHIFT ],
-  Q => [ SDLK_q, SDL::KMOD_SHIFT ],
-  R => [ SDLK_r, SDL::KMOD_SHIFT ],
-  S => [ SDLK_s, SDL::KMOD_SHIFT ],
-  T => [ SDLK_t, SDL::KMOD_SHIFT ],
-  U => [ SDLK_u, SDL::KMOD_SHIFT ],
-  V => [ SDLK_v, SDL::KMOD_SHIFT ],
-  W => [ SDLK_w, SDL::KMOD_SHIFT ],
-  X => [ SDLK_x, SDL::KMOD_SHIFT ],
-  Y => [ SDLK_y, SDL::KMOD_SHIFT ],
-  Z => [ SDLK_z, SDL::KMOD_SHIFT ],
+  A => [ SDLK_a, KMOD_SHIFT ],
+  B => [ SDLK_b, KMOD_SHIFT ],
+  C => [ SDLK_c, KMOD_SHIFT ],
+  D => [ SDLK_d, KMOD_SHIFT ],
+  E => [ SDLK_e, KMOD_SHIFT ],
+  F => [ SDLK_f, KMOD_SHIFT ],
+  G => [ SDLK_g, KMOD_SHIFT ],
+  H => [ SDLK_h, KMOD_SHIFT ],
+  I => [ SDLK_i, KMOD_SHIFT ],
+  J => [ SDLK_j, KMOD_SHIFT ],
+  K => [ SDLK_k, KMOD_SHIFT ],
+  L => [ SDLK_l, KMOD_SHIFT ],
+  M => [ SDLK_m, KMOD_SHIFT ],
+  N => [ SDLK_n, KMOD_SHIFT ],
+  O => [ SDLK_o, KMOD_SHIFT ],
+  P => [ SDLK_p, KMOD_SHIFT ],
+  Q => [ SDLK_q, KMOD_SHIFT ],
+  R => [ SDLK_r, KMOD_SHIFT ],
+  S => [ SDLK_s, KMOD_SHIFT ],
+  T => [ SDLK_t, KMOD_SHIFT ],
+  U => [ SDLK_u, KMOD_SHIFT ],
+  V => [ SDLK_v, KMOD_SHIFT ],
+  W => [ SDLK_w, KMOD_SHIFT ],
+  X => [ SDLK_x, KMOD_SHIFT ],
+  Y => [ SDLK_y, KMOD_SHIFT ],
+  Z => [ SDLK_z, KMOD_SHIFT ],
 
   0 => SDLK_0,
   1 => SDLK_1,
